@@ -57,7 +57,7 @@ class Build:
                 refspec = build_row['revision']
             # Make a local clone of the repository from origin so we don't have to clone entire repository every time
             if not os.path.exists(self.src_repo):
-                if self._shell_exec("git clone --progress git@%s %s" % (aegis.config.get('git_repo'), options.program_name), cwd=self.src_dir, build_step='build'):
+                if self._shell_exec("git clone --progress %s %s" % (aegis.config.get('git_repo'), options.program_name), cwd=self.src_dir, build_step='build'):
                     return
             # Fetch all the changes to repo and set the correct branch and revision
             if self._shell_exec("git fetch --all", cwd=self.src_repo, build_step='build'):
@@ -120,11 +120,18 @@ class Build:
                     build_size = os.path.getsize(build_output_file)
                     if build_size:
                         self.build_row.set_build_size(build_size)
-            # Rsync the files to the servers if it's configured
+            # Rsync the files to the servers if it's configured, using non-blocking multi-shell so the rsync run in parallel
             rsync_hosts = [rh for rh in aegis.config.get('deploy_hosts') if rh != self.host]
+            cmds = []
             for rsync_host in rsync_hosts:
-                cmd = "nice rsync -q --password-file=/etc/rsync.password -avzhW %s www-data@%s::%s" % (self.build_dir, rsync_host, options.rsync_module)
-                if self._shell_exec(cmd, cwd=self.build_dir, build_step='build'):
+                cmd = "nice rsync -q --password-file=/etc/rsync.password -avhW %s www-data@%s::%s" % (self.build_dir, rsync_host, options.rsync_module)
+                cmds.append(cmd)
+            for proc in aegis.stdlib.multi_shell(cmds, cwd=self.build_dir):
+                stdout, stderr = proc.communicate()
+                stdout = stdout.decode('utf-8').strip()
+                stderr = stderr.decode('utf-8').strip()
+                cmd = ' '.join(proc.args)
+                if self._shell_exec(cmd, build_step='build', cwd=self.build_dir, stdout=stdout, stderr=stderr, exit_status=proc.returncode):
                     return
         except Exception as ex:
             logging.exception(ex)
@@ -154,7 +161,22 @@ class Build:
         num_procs = 1 if main_is_hydra else 0
         if len(processes) == num_procs:
             aegis.stdlib.loge(processes, "No processes ending with _%s to restart." % env)
-        # Remove and re-link
+        # Set up a set of <env>_prev-# to use with try_files in nginx
+        prev_version = self.build_row['previous_version']
+        if prev_version:
+            for prev_num in range(1, 6):
+                link_file = os.path.join(app_dir, '%s_prev-%s' % (env, prev_num))
+                link_target = os.path.join(app_dir, prev_version)
+                if self._shell_exec("rm -f %s" % link_file, build_step=build_step, cwd=app_dir):
+                    return
+                if self._shell_exec("ln -s %s %s" % (link_target, link_file), build_step=build_step, cwd=app_dir):
+                    return
+                # Get previous build and its previous version
+                prev_build = aegis.model.Build.get_version(prev_version)
+                prev_version = prev_build['previous_version']
+                if not prev_version:
+                    break
+        # GO LIVE
         if self._shell_exec("rm -f %s" % live_symlink, build_step=build_step, cwd=app_dir):
             return
         if self._shell_exec("ln -s %s %s" % (build_dir, live_symlink), build_step=build_step, cwd=app_dir):
@@ -203,11 +225,12 @@ class Build:
             self.logw("IS ALL DELETED")
 
 
-
     # Shell execution with structured logging and database output handling.
-    def _shell_exec(self, cmd, cwd, build_step, env=None):
+    def _shell_exec(self, cmd, cwd, build_step, env=None, stdout=None, stderr=None, exit_status=None):
         exec_output = "%s@%s:%s> %s" % (self.username, self.host, cwd, cmd)
-        stdout, stderr, exit_status = aegis.stdlib.shell(cmd, cwd=cwd, env=env)
+        # Main case is that we're executing. But alternate is to execute elsewhere and pass in the outputs
+        if exit_status is None:
+            stdout, stderr, exit_status = aegis.stdlib.shell(cmd, cwd=cwd, env=env)
         if stdout:
             exec_output += '\n%s' % stdout
         if stderr:
@@ -272,9 +295,12 @@ class Build:
         app_dir = os.path.join(options.deploy_dir, options.program_name)
         build_dir = os.path.join(app_dir, build_row['version'])
         live_build = aegis.model.Build.get_live_build(build_row['env'])
-        commits, stderr, exit_status = aegis.stdlib.shell('git log --oneline --decorate %s..%s' % (live_build['version'], build_row['version']), cwd=build_dir)
-        commits = commits.splitlines()
-        return commits
+        if live_build:
+            commits, stderr, exit_status = aegis.stdlib.shell('git log --oneline --decorate %s..%s' % (live_build['version'], build_row['version']), cwd=build_dir)
+            commits = commits.splitlines()
+            return commits
+        else:
+            return []
 
 
     # Set previous version and send notifications
